CREATE OR REPLACE PACKAGE BODY GIS_AR_INTERFACE AS

/*-------------------------------------------------------------------------+
 | PUBLIC PROCEDURE                                                        |
 |    customer_exists                                                      |
 |                                                                         |
 | DESCRIPTION                                                             |
 |    This is a stub module for global imaging to check if a customer      |
 |    exists.                                                              |
 |                                                                         |
 | ARGUMENTS                                                               |
 |    n_gtn_id                GTN ID                                       |
 |    v_customer_number       Customer number to validate                  |
 |    v_customer_name         Customer name                                |
 |    n_status                Return status as:                            |
 |                            -1     Error encountered                     |
 |                            0      Customer does not exist               |
 |                            1      Customer exists, STATUS = 'A'ctive    |
 |                            2      Customer exists, STATUS = 'I'nactive  |
 |    v_err                   Error message                                |
 |                                                                         |
 | RETURNS                                                                 |
 |                                                                         |
 | NOTES                                                                   |
 |                                                                         |
 | EXAMPLE                                                                 |
 |    customer_exists(743, '270380600', v_customer_name, n_status, v_err)  |
 |                                                                         |
 | MODIFICATION HISTORY                                                    |
 |    31-May-2000      Patrick Low     Created.                            |
 |    01-Jun-2000      Patrick Low     Convert parameters to Hungarian     |
 |                                     notation, add event logging         |
 |                                                                         |
 +-------------------------------------------------------------------------*/

PROCEDURE customer_exists (n_gtn_id          IN  NUMBER,
                           v_customer_number IN  VARCHAR2,
                           v_customer_name   OUT VARCHAR2,
                           n_status          OUT NUMBER,
                           v_err             OUT VARCHAR2) IS
  n_org_id    NUMBER;
BEGIN

  /* Audit event */

  select to_number(decode(substr(userenv('CLIENT_INFO'),1,1),' ',null,substr(userenv('CLIENT_INFO'),1,10)))
  into   n_org_id
  from   dual;

  gis_common.gis_audit_event(n_org_id,
                             n_gtn_id,
                            'GIS_AR_INTERFACE.CUSTOMER_EXISTS',
                             To_char(n_gtn_id) ||','||v_customer_number,
                            'CHECK CUSTOMER EXISTS',
                            SQLERRM,
                            v_err);

  /* Check if customer exists and is active */
  SELECT  decode(status, 'A', 1, 2), customer_name
  INTO    n_status, v_customer_name
  FROM    ra_customers
  WHERE   customer_number = v_customer_number;

  v_err:= NULL;

  /* Test exception handler */
  --raise_application_error(-20000,'Test error');

EXCEPTION

  WHEN NO_DATA_FOUND THEN
    /* Return default value of 0 */
    n_status := 0;
    v_err := SQLERRM;

  WHEN OTHERS THEN
    n_status := -1;
    v_err := SQLERRM;

END customer_exists;

/*-------------------------------------------------------------------------+
 | PUBLIC PROCEDURE                                                        |
 |    create_customer                                                      |
 |                                                                         |
 | DESCRIPTION                                                             |
 |    This is a stub module for global imaging to create a customer via    |
 |    the open interface tables.                                           |
 |                                                                         |
 | ARGUMENTS                                                               |
 |    n_org_id                Organization ID                              |
 |    n_gtn_id                GTN ID                                       |
 |    n_gis_new_customer_id   Unique sequence number from GIS source       |
 |    n_gis_status            Return status of customer interface process  |
 |    d_gis_status_date       Date of customer interface process           |
 |    v_gis_customer_name     Name of new customer                         |
 |    v_err                   Error message                                |
 |                                                                         |
 | RETURNS                                                                 |
 |                                                                         |
 | NOTES                                                                   |
 |                                                                         |
 | EXAMPLE                                                                 |
 |    create_customer(4074, 38, 68, n_status, d_status_date, 'ACME', v_err)|
 |                                                                         |
 | MODIFICATION HISTORY                                                    |
 |    31-May-2000      Patrick Low     Created.                            |
 |    01-Jun-2000      Patrick Low     Convert parameters to Hungarian     |
 |                                     notation, add event logging         |
 |                                                                         |
 +-------------------------------------------------------------------------*/

PROCEDURE create_customer (n_org_id               IN  NUMBER,
                           n_gtn_id               IN  NUMBER,
                           n_gis_new_customer_id  IN  NUMBER,
                           n_gis_status           OUT NUMBER,
                           d_gis_status_date      OUT DATE,
                           v_gis_customer_name    IN  VARCHAR2,
                           v_err                  OUT VARCHAR2) IS

  v_customer_status         VARCHAR2(1) := 'A';
  v_gis_customer_inf_code   VARCHAR2(1) := 'I';
  n_gis_default_user_id     NUMBER(15) := 3;
  v_default_cust_profile    VARCHAR2(30) := 'DEFAULT';
  v_credit_hold             VARCHAR2(1) := 'N';

BEGIN

  /* Get default lookup values */

--  gis_common.gis_lookup('GIS_DEFAULTS', 'ACTIVE CUSTOMER', v_customer_status, v_err);
--  gis_common.gis_lookup('GIS_DEFAULTS', 'INSERT FLAG', v_gis_customer_inf_code, v_err);
--  gis_common.gis_lookup('GIS_DEFAULTS', 'INSERT UPDATE USER ID', n_gis_default_user_id, v_err);
--  gis_common.gis_lookup('GIS_DEFAULTS', 'CUSTOMER PROFILE', v_default_cust_profile, v_err);
--  gis_common.gis_lookup('GIS_DEFAULTS', 'CREDIT HOLD', v_credit_hold, v_err);
    n_gis_status := 30;
--  gis_common.gis_lookup('GIS_STATUS_CODE', 'INTERFACE CUSTOMER', n_gis_status, v_err);

  INSERT INTO ra_customers_interface_all
    (org_id,
     orig_system_customer_ref,
     customer_name,
     customer_status,
     insert_update_flag,
     last_updated_by,
     last_update_date,
     created_by,
     creation_date)
  VALUES
    (n_org_id,
     n_gis_new_customer_id,
     v_gis_customer_name,
     v_customer_status,
     v_gis_customer_inf_code,
     n_gis_default_user_id,
     sysdate,
     n_gis_default_user_id,
     sysdate);

  INSERT INTO ra_customer_profiles_int_all
    (org_id,
     orig_system_customer_ref,
     insert_update_flag,
     customer_profile_class_name,
     credit_hold,
     last_updated_by,
     last_update_date,
     created_by,
     creation_date)
  VALUES
    (n_org_id,
     n_gis_new_customer_id,
     v_gis_customer_inf_code,
     v_default_cust_profile,
     v_credit_hold,
     n_gis_default_user_id,
     sysdate,
     n_gis_default_user_id,
     sysdate);

  d_gis_status_date := SYSDATE;
  v_err:= NULL;

  /* Test exception handler */
  --raise_application_error(-20000,'Test error');

EXCEPTION

  WHEN OTHERS THEN
    v_err := SQLERRM;

END create_customer;

END GIS_AR_INTERFACE;
/