CREATE OR REPLACE PACKAGE BODY APPS.GIS_COMMON AS

/*-------------------------------------------------------------------------+
 | PUBLIC PACKAGE                                                          |
 |   gis_common                                                            |
 |                                                                         |
 | MODIFICATION HISTORY                                                    |
 |   31-May-2000      Patrick Low     Created.                             |
 |   19-Oct-2000      Patrick Low     Updated to be created in APPS schema |
 |                                                                         |
 +-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------+
 | PUBLIC PROCEDURE                                                        |
 |   gis_lookup                                                            |
 |                                                                         |
 | DESCRIPTION                                                             |
 |   This is a stub module for global imaging to perform lookups           |
 |                                                                         |
 | ARGUMENTS                                                               |
 |   v_lookup_type           Lookup type                                   |
 |   v_meaning               Meaning of lookup code                        |
 |   v_lookup_code           Lookup code                                   |
 |   v_err                   Error message                                 |
 |                                                                         |
 | RETURNS                                                                 |
 |                                                                         |
 | NOTES                                                                   |
 |                                                                         |
 | EXAMPLE                                                                 |
 |   gis_lookup('GIS_STATUS_CODE', 'UNPROCESSED', v_lookup_code, v_err)    |
 |                                                                         |
 | MODIFICATION HISTORY                                                    |
 |   31-May-2000      Patrick Low     Created.                             |
 |                                                                         |
 +-------------------------------------------------------------------------*/

PROCEDURE gis_lookup (v_lookup_type  IN  VARCHAR2,
                      v_meaning      IN  VARCHAR2,
                      v_lookup_code  OUT VARCHAR2,
                      v_err          OUT VARCHAR2) IS
BEGIN

  /* Check for lookup code */
  SELECT  lookup_code
  INTO    v_lookup_code
  FROM    fnd_lookups
  WHERE   lookup_type = v_lookup_type
  AND     meaning = v_meaning;

  v_err:= NULL;

  /* Test exception handler */
  --raise_application_error(-20000,'Test error');

EXCEPTION

  WHEN NO_DATA_FOUND THEN
    v_lookup_code := NULL;
    v_err := SQLERRM;

  WHEN OTHERS THEN
    v_lookup_code := NULL;
    v_err := SQLERRM;

END gis_lookup;

/*-------------------------------------------------------------------------+
 | PUBLIC PROCEDURE                                                        |
 |   gis_audit_event                                                       |
 |                                                                         |
 | DESCRIPTION                                                             |
 |   This is a stub module for global imaging to audit events              |
 |                                                                         |
 | ARGUMENTS                                                               |
 |   n_org_id                Organization ID                               |
 |   n_gtn_id                GTN ID                                        |
 |   v_object_name           Source object name of error                   |
 |   v_input_parameters      Parameters in call                            |
 |   v_error_event           Event in object                               |
 |   v_error_message         Oracle error message                          |
 |   v_err                   Error message                                 |
 |                                                                         |
 | RETURNS                                                                 |
 |                                                                         |
 | NOTES                                                                   |
 |                                                                         |
 | EXAMPLE                                                                 |
 |   gis_audit_event(4074,                                                 |
 |                   1002,                                                 |
 |                   'GIS_AR_INTERFACE.CREATE_CUSTOMER',                   |
 |                   '4074, ACME PTY LTD, A, Y'                            |
 |                   'INSERTING INTO RA_CUSTOMER_INTERFACE_ALL',           |
 |                   'ORA-09214: Oracle error encountered',                |
 |                   vErr)                                                 |
 |                                                                         |
 | MODIFICATION HISTORY                                                    |
 |   01-June-2000      Patrick Low     Created.                            |
 |                                                                         |
 +-------------------------------------------------------------------------*/

PROCEDURE gis_audit_event (n_org_id           IN  NUMBER,
                           n_gtn_id           IN  NUMBER,
                           v_object_name      IN  VARCHAR2,
                           v_input_parameters IN VARCHAR2,
                           v_error_event      IN VARCHAR2,
                           v_error_message    IN VARCHAR2,
                           v_err              OUT VARCHAR2) IS

  n_gis_default_user_id     NUMBER(15) := 3;

BEGIN

  /* Get default lookup values */
--  gis_common.gis_lookup('GIS_DEFAULTS', 'INSERT UPDATE USER ID', n_gis_default_user_id, v_err);

  INSERT INTO gis_event_log_all
    (event_id,
     gtn_id,
     object_name,
     input_parameters,
     error_event,
     error_message,
     last_updated_by,
     last_update_date,
     last_update_login,
     created_by,
     creation_date,
     org_id)
  VALUES
    (gis_event_log_s.nextval,
     n_gtn_id,
     v_object_name,
     v_input_parameters,
     v_error_event,
     v_error_message,
     n_gis_default_user_id,
     SYSDATE,
     NULL,
     n_gis_default_user_id,
     SYSDATE,
     n_org_id);

  v_err:= NULL;

  /* Test exception handler */
  --raise_application_error(-20000,'Test error');

EXCEPTION

  WHEN NO_DATA_FOUND THEN
    v_err := SQLERRM;

  WHEN OTHERS THEN
    v_err := SQLERRM;

END gis_audit_event;

END GIS_COMMON;
/