--  File Name : gis_load_vs_doc_pkg.sql                                       -
-- 				                                                              -
--  Purpose   : package used to migrate data from GIS v1.0                    -
--              into GIS 2.0 R11i. Is called from program that inserts        -
--				copied images from the old system into the table fnd_lobs     -
--              This package then attaches these images to the required       -
--				application records in 11i using the id values stored in the  -
-- 				temp table gis_lobs_import                                    -
--                                                                            -
--  Arguments :  p_file_id IN NUMBER                                          -
--				 p_file_name IN VARCHAR2                                      -
--				 p_vs_docid IN NUMBER);	                                      -
--                                                                            -
--  History                                                                   -
--                                                                            -
--  Date      Name            Comments                                        -
--                                                                            -
--  14/06/00  David Gough    Initial version                                  -
--  30/06/00  David Gough    Tested and working on HRDEV                      -
--  29/11/00  David Gough    Upgraded for latest attachment entities          -
--  03/01/01  David Gough    Updated to handle GSR documents without          -
--							 Associated finacial record						  - 
-------------------------------------------------------------------------------
CREATE OR REPLACE
PACKAGE GIS_LOAD_VS_DOC AS

PROCEDURE load_vs_doc (	p_file_id IN NUMBER
					    ,p_file_name IN VARCHAR2
					    ,p_vs_docid IN NUMBER);								
								
END GIS_LOAD_VS_DOC;
/

CREATE OR REPLACE
PACKAGE BODY GIS_LOAD_VS_DOC AS

PROCEDURE load_vs_doc( p_file_id IN NUMBER, p_file_name IN VARCHAR2, p_vs_docid IN NUMBER) IS

CURSOR cur_fnd_doc_cats( f_vs_doc_type VARCHAR) IS
SELECT DISTINCT category_id, user_name, attribute1
FROM
	fnd_document_categories_vl
WHERE name = f_vs_doc_type
AND attribute_category = 'GIS';
	 
CURSOR cur_fnd_lobs_import( f_vs_docid NUMBER) IS
SELECT DISTINCT
	vs_docid,
	doc_type,
	agreement_id,
	header_id,
	customer_id,
	project_id,
	invoice_id,
	gsr_id
FROM
	gis_migrate_check
WHERE 
	vs_docid = f_vs_docid;

CURSOR cur_fnd_lobs_vs_docid( f_vs_docid VARCHAR) IS
SELECT DISTINCT
	file_id,
	file_name,
    program_tag  	
FROM
	fnd_lobs
WHERE
	program_tag = f_vs_docid
AND
	program_name = 'GIS_MIGRATE'
ORDER BY file_id;

CURSOR cur_okc_headers( f_gsr_id NUMBER) IS
SELECT DISTINCT id
FROM okc_k_headers_b
WHERE attribute1 = f_gsr_id;

-- cursor rowtypes
row_okc_headers cur_okc_headers%ROWTYPE;	
row_fnd_lobs_import cur_fnd_lobs_import%ROWTYPE;
row_fnd_doc_cats cur_fnd_doc_cats%ROWTYPE;
row_fnd_lobs_vs_docid cur_fnd_lobs_vs_docid%ROWTYPE;

-- local variables
l_rowid VARCHAR2(30);
l_attached_document_id NUMBER;
l_document_id		NUMBER;	
l_creation_date	DATE := SYSDATE;
l_created_by		NUMBER:=1;
l_last_update_date	DATE := SYSDATE;
l_last_updated_by	NUMBER:=1;
l_lang			VARCHAR2(40);
l_seq_num NUMBER;		-- sequency number for attached records
l_counter NUMBER;
l_insert_flag BOOLEAN := FALSE;
l_entity_name VARCHAR2(30) := NULL;
l_pk1_value NUMBER(10);
l_datatype_id NUMBER := 6;  -- 6 = 'file' datatype
l_category_id NUMBER;
l_document_description VARCHAR2(255);
l_base_document_description VARCHAR2(255);
l_media_id NUMBER(10);
l_security_type NUMBER(2) := 4; -- 4 = none
l_security_id NUMBER(10) := 1001; -- US org org_id
l_publish_flag CHAR := 'N';
l_storage_location VARCHAR2(25) := 'TBA';
l_us_set_of_books_id NUMBER(15) := 100126;
l_us_org_id NUMBER(15) := 1001;
nContractCount NUMBER(15);
l_attribute_category VARCHAR2(50) := NULL;
l_attribute1 VARCHAR2(50) := NULL;
l_attribute2 VARCHAR2(50) := NULL;
l_attribute3 VARCHAR2(50) := NULL;
l_attribute6 VARCHAR2(50) := NULL;
l_custName VARCHAR2(50) := NULL;
l_custNum  VARCHAR2(50) := NULL;
 
BEGIN
	l_media_id := p_file_id;    
	IF NOT cur_fnd_lobs_import%ISOPEN 
	THEN
		OPEN cur_fnd_lobs_import( p_vs_docid);
	END IF;
	
	FETCH cur_fnd_lobs_import INTO row_fnd_lobs_import;
	IF cur_fnd_lobs_import%FOUND
	THEN
--	dbms_output.put_line('Record found');
-- get the current user LANG
		SELECT userenv('LANG')
		INTO l_lang
		FROM DUAL;

-- get the category id for the vs_doc_type
		IF NOT cur_fnd_doc_cats%ISOPEN 
		THEN
			dbms_output.put_line(row_fnd_lobs_import.doc_type);
			OPEN cur_fnd_doc_cats( row_fnd_lobs_import.doc_type);
		END IF;
		
		FETCH cur_fnd_doc_cats INTO row_fnd_doc_cats;
		
		IF NOT cur_fnd_doc_cats%FOUND 
		THEN
-- default category to CUMISC
			SELECT category_id
			INTO l_category_id 
			FROM fnd_document_categories
			WHERE name =  'CUMISC';
			
			l_document_description := 'GIS MIGRATED - MISC.';
			l_security_type := 2;
			l_security_id := l_us_set_of_books_id;
			l_publish_flag := 'N';
			l_entity_name := 'AR_CUSTOMERS';
			l_pk1_value :=	row_fnd_lobs_import.customer_id;
		ELSE
-- use the category user name as the document description and note
-- that it is a GIS migrated doc
			l_category_id :=row_fnd_doc_cats.category_id;
			l_document_description := 'GIS MIGRATED';
-- set the required security fields depending on the document context
			IF row_fnd_doc_cats.attribute1 = 'ORDER MANAGEMENT'
			THEN
				l_security_type := 1; -- org_id
				l_publish_flag := 'N';
				l_security_id := l_us_org_id;					
				l_entity_name := 'OE_ORDER_HEADERS';
				l_pk1_value :=	row_fnd_lobs_import.header_id;				
			ELSIF row_fnd_doc_cats.attribute1 = 'CONTRACTS'
			THEN
				l_security_type := 1;
				l_publish_flag := 'N';
				l_security_id := l_us_org_id;
				l_entity_name := 'OKC_K_HEADERS_V';
					
				-- could be more than one record here
--				SELECT count(id)
--				INTO nContractCount
--				FROM okc_k_headers_b
--				WHERE attribute1 = row_fnd_lobs_import.gsr_id;

	-- need to handle documents that have no related finacial record
				
				BEGIN
					SELECT party_name
					, party_number
					INTO l_custName
					,l_custNum
					FROM hz_parties
					WHERE party_id = row_fnd_lobs_import.customer_id;
				EXCEPTION
					WHEN OTHERS
					THEN
						NULL;
				END;
				
				l_attribute_category := 'Old Renewal';
				l_attribute1 := l_custName;
				l_attribute2 := l_custNum;
				l_attribute3 := row_fnd_lobs_import.gsr_id;
				l_attribute6 := 'COMPLETED';
				
			ELSIF row_fnd_doc_cats.attribute1 = 'RECEIVABLES'
			THEN
				l_security_type := 2; -- set of books
				l_publish_flag := 'N';
				l_security_id := l_us_set_of_books_id;
				l_entity_name := 'RA_CUSTOMER_TRX';
				l_pk1_value :=	row_fnd_lobs_import.invoice_id;	
			ELSIF row_fnd_doc_cats.attribute1 = 'AGREEMENTS'
			THEN
				IF row_fnd_lobs_import.doc_type = 'CSA'
				THEN
					l_security_type := 2;
					l_publish_flag := 'N';
					l_security_id := l_us_set_of_books_id;
					l_entity_name := 'AR_CUSTOMERS';
					l_pk1_value :=	row_fnd_lobs_import.customer_id;
					
					SELECT category_id
					INTO l_category_id 
					FROM fnd_document_categories
					WHERE name =  'MSA';
					
				ELSE	
					l_security_type := 1;-- org_id
					l_publish_flag := 'Y';
					l_security_id := l_us_org_id;
					l_entity_name := 'OE_AGREEMENTS';
					l_pk1_value :=	row_fnd_lobs_import.agreement_id;
				END IF;			
			ELSIF row_fnd_doc_cats.attribute1 = 'CUSTOMER'
			THEN
				l_security_type := 2;
				l_publish_flag := 'N';
				l_security_id := l_us_set_of_books_id;
				l_entity_name := 'AR_CUSTOMERS';
				l_pk1_value :=	row_fnd_lobs_import.customer_id;			
			ELSIF row_fnd_doc_cats.attribute1 = 'PROJECTS'
			THEN
				l_security_type := 1;-- org_id
				l_publish_flag := 'Y';
				l_security_id := l_us_org_id;
				l_entity_name := 'PA_PROJECTS';
				l_pk1_value :=	row_fnd_lobs_import.project_id;						
			END IF;						
		END IF;
		
		CLOSE cur_fnd_doc_cats;

	-- gis_document_pkg inserts the required details into the tables
	-- fnd_documents and fnd_documents_tl. The document_id is returned from
	-- the package for use in fnd__attached_documents 
		gis_documents_pkg.Insert_Row(
	            X_rowid               => l_rowid,
	            X_document_id         => l_document_id,
	            X_creation_date       => l_creation_date,
	            X_created_by          => l_created_by,
	            X_last_update_date    => l_last_update_date,
	            X_last_updated_by     => l_last_updated_by,
	            X_last_update_login   => NULL,
	            X_datatype_id         => l_datatype_id,
	            X_category_id         => l_category_id,
	            X_security_type       => l_security_type,
	            X_security_id         => l_security_id,
	           X_publish_flag         => l_publish_flag,
	           X_image_type           => NULL,
	           X_storage_type         => NULL,
	           X_usage_type           => 'S',
	           X_start_date_active    =>  SYSDATE,
	           X_end_date_active      => NULL,
	           X_request_id           => NULL,
	           X_program_application_id => NULL,
	           X_program_id           => NULL,
	           X_program_update_date  => NULL,
	           X_language             => l_lang,
	           X_description          => l_document_description,
	           X_file_name            => 'INTERNAL',
	           X_media_id             => l_media_id,
		   	   X_attribute_category   => l_attribute_category,
		       X_attribute1           => l_attribute1,
		       X_attribute2           => l_attribute2,
		       X_attribute3           => l_attribute3,
		       X_attribute4           => NULL,
		       X_attribute5           => NULL,
		       X_attribute6           => l_attribute6,
		       X_attribute7           => NULL,
		       X_attribute8           => NULL,
		       X_attribute9           => NULL,
		       X_attribute10          => l_storage_location,
		       X_attribute11          => NULL,
		       X_attribute12          => NULL,
		       X_attribute13          => NULL,
		       X_attribute14          => NULL,
		       X_attribute15          => NULL);  		
			
			IF l_entity_name IS NOT NULL
			THEN
	-- if its a contracts record and there is more than one loop
				IF l_entity_name = 'OKC_K_HEADERS_V'
				THEN
					IF NOT cur_okc_headers%ISOPEN 
					THEN
						OPEN cur_okc_headers( row_fnd_lobs_import.gsr_id);
					END IF;
					
					FETCH cur_okc_headers INTO row_okc_headers;
					WHILE cur_okc_headers%FOUND
					LOOP
						l_pk1_value := row_okc_headers.id;
			
			-- Attached Document Id has to be populated from the sequence.
						SELECT fnd_attached_documents_s.nextval
						INTO l_attached_document_id
						FROM sys.dual;
		
			-- the sequence number is a sequence identifier for each record
			-- attached to a particular document entity for a specific financials
			-- record. It starts at 10 and icreases by 10 for each new row.
						SELECT MAX(seq_num)+10
						INTO l_seq_num
						FROM fnd_attached_documents
						WHERE entity_name = l_entity_name
						AND pk1_value = l_pk1_value; 
						
						IF l_seq_num IS NULL
						THEN
							l_seq_num := 10;
						END IF;					

			-- gis_attached_documents inserts a row into the table
			-- fnd_attached_documents					
					  	gis_attached_documents_pkg.insert_row (
						x_attached_document_id	=> l_attached_document_id,
						x_document_id		=> l_document_id,
						x_creation_date		=> l_creation_date,
						x_created_by		=> l_created_by	,
						x_last_update_date	=> l_last_update_date,
						x_last_updated_by	=> l_last_updated_by,
						x_last_update_login	=> NULL	,
						x_seq_num		=> l_seq_num,
						x_entity_name		=> l_entity_name,
						x_pk1_value		=> l_pk1_value	,
						x_pk2_value		=> NULL			,
						x_pk3_value		=> NULL			,
						x_pk4_value		=> NULL			,
						x_pk5_value		=> NULL			,
						x_automatically_added_flag	=> 'N',
						x_attribute_category	=> NULL	,
						x_attribute1		=> NULL	,
						x_attribute2		=> NULL	,
						x_attribute3		=> NULL	,
						x_attribute4		=> NULL	,
						x_attribute5		=> NULL	,
						x_attribute6		=> NULL	,
						x_attribute7		=> NULL	,
						x_attribute8		=> NULL	,
						x_attribute9		=> NULL	,
						x_attribute10		=> NULL	,
						x_attribute11		=> NULL	,
						x_attribute12		=> NULL	,
						x_attribute13		=> NULL	,
						x_attribute14		=> NULL	,
						x_attribute15		=> NULL	);	
										
						FETCH cur_okc_headers INTO row_okc_headers;
					END LOOP;
					CLOSE cur_okc_headers;	
				ELSE
		-- Attached Document Id has to be populated from the sequence.
					SELECT fnd_attached_documents_s.nextval
					INTO l_attached_document_id
					FROM sys.dual;
	
		-- the sequence number is a sequence identifier for each record
		-- attached to a particular document entity for a specific financials
		-- record. It starts at 10 and icreases by 10 for each new row.
					SELECT MAX(seq_num)+10
					INTO l_seq_num
					FROM fnd_attached_documents
					WHERE entity_name = l_entity_name
					AND pk1_value = l_pk1_value; 

					IF l_seq_num IS NULL
					THEN
						l_seq_num := 10;
					END IF;	
													
		-- gis_attached_documents inserts a row into the table
		-- fnd_attached_documents					
				  	gis_attached_documents_pkg.insert_row (
					x_attached_document_id	=> l_attached_document_id,
					x_document_id		=> l_document_id,
					x_creation_date		=> l_creation_date,
					x_created_by		=> l_created_by	,
					x_last_update_date	=> l_last_update_date,
					x_last_updated_by	=> l_last_updated_by,
					x_last_update_login	=> NULL	,
					x_seq_num		=> l_seq_num,
					x_entity_name		=> l_entity_name,
					x_pk1_value		=> l_pk1_value	,
					x_pk2_value		=> NULL			,
					x_pk3_value		=> NULL			,
					x_pk4_value		=> NULL			,
					x_pk5_value		=> NULL			,
					x_automatically_added_flag	=> 'N',
					x_attribute_category	=> NULL	,
					x_attribute1		=> NULL	,
					x_attribute2		=> NULL	,
					x_attribute3		=> NULL	,
					x_attribute4		=> NULL	,
					x_attribute5		=> NULL	,
					x_attribute6		=> NULL	,
					x_attribute7		=> NULL	,
					x_attribute8		=> NULL	,
					x_attribute9		=> NULL	,
					x_attribute10		=> NULL	,
					x_attribute11		=> NULL	,
					x_attribute12		=> NULL	,
					x_attribute13		=> NULL	,
					x_attribute14		=> NULL	,
					x_attribute15		=> NULL	);
				END IF;  
		-- Commit the transaction
		 		COMMIT;
	 		
	 			l_entity_name := NULL;
		
			END IF;
		CLOSE cur_fnd_lobs_import;
	
	END IF;
			
EXCEPTION

  WHEN OTHERS THEN
		DECLARE
		errorCode NUMBER := SQLCODE;
		errorMsg VARCHAR(300) := SQLERRM;
		BEGIN
			DBMS_OUTPUT.put_line ( errorMsg ||'  ' || TO_CHAR( errorCode));     
			IF NOT cur_fnd_lobs_import%ISOPEN 
			THEN
				CLOSE cur_fnd_lobs_import;
			END IF;
			
			IF NOT cur_fnd_doc_cats%ISOPEN 
			THEN
				CLOSE cur_fnd_doc_cats;
			END IF;
			
			IF NOT cur_fnd_lobs_vs_docid%ISOPEN 
			THEN
				CLOSE cur_fnd_lobs_vs_docid;
			END IF;
		END;
				
END load_vs_doc;

END GIS_LOAD_VS_DOC;  
/
