CREATE OR REPLACE PACKAGE BODY APPS.GIS_OM_INTERFACE AS
                                                                          
                                                                                   
/*-------------------------------------------------------------------------+
Setups:
   1. create order_source_id's for each org's order_import requests. use the applications/forms for this,
      so that it will assign us [unique] id's:
         SELECT order_source_id
         FROM so_order_sources
         WHERE name = (
            SELECT 'GIS - ' ||name
            FROM hr_operating_units
            WHERE organization_id = p_org_id );
   2. Concurrent Request Set:
      1) exec gis_om_interface.gis_order_request 
      2) call order_import program using correct order_source_id [see above]
      3) exec gis_om_interface.gis_return_oenum
          
      
TBD:                                                     
   1. check if duplicate request
      a) if duplicate request, set interface_status = '62' ["DUPLICATE OE IFACE REQUEST"]; don't 
         populate oe_iface.
      b) later, when retrieving, we must parse '62's for matches w/ newly-retrieved oe#'s, and
         finalize them as well.
   2. copy the GTN to oe_order_headers
   3. ALERTS:
      a) parse gis_interface for interface_status = '55' -> ERROR
      b) parse gis_interface for last_update_date > ????. 
      
      
                                                                         
GIS_INTERFACE.interface_status :=      
         10   UNPROCESSED
         20   CREATE CUSTOMER
         30   CUSTOMER INTERFACE ELIGIBLE
         35   CUSTOMER INTERFACE ERROR
         40   CUSTOMER INTERFACE COMPLETE
         50   CUSTOMER EXISTS              
         55   ORDER INTERFACE ERROR
         60   ORDER INTERFACE RECORD INSERTED
         70 - ORDER INTERFACE COMPLETE
         71 - DOCUMENT ATTACHED SUCCESSFULLY'
         72 - UPDATE GIS_DOC_STATUS -> "PRE-INDEXED"
         73 - UPDATE FND_DOCUMENTS_TL
         74 - INSERT OE# INTO OE_ORDER_HEADERS_ALL (_DFV)
         80 - DONE.
                                                                           
 +-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------+
 | PUBLIC PROCEDURE                                                        |
 |    gis_order_request                                                    |
 |                                                                         |
 | DESCRIPTION                                                             |
 |    This is a Concurrent Process to                
       (a) determine if there are requests for Orders 
           in gis_interface, and if so 
       (b) call gis_create_order.  |
 |                                                                         |
 | MODIFICATION HISTORY                                                    |
 |    02-Jun-2000       Dave Anselmi  Created.                             |
 |    15-Oct-2000       Dave Anselmi  Exception Handling.                  |
 |    21-Jan-2001       Dave Anselmi  Complete/Installed in AU             |
 |    02-Apr-2001       Patrick Low   Remove references to _all tables     |
 |    22-Jun-2001       Dave Anselmi  Added ROLLBACK & COMMIT to EXCEPTION |
 |    03-Jul-2001       Dave Anselmi  Made code multi-attach aware         |
 |                                                                         |
 +-------------------------------------------------------------------------*/

PROCEDURE gis_order_request (errbuf   OUT VARCHAR2,
                             retcode  OUT VARCHAR2)

IS

   CURSOR       cInterface_Program IS
      SELECT    *  
      FROM      gis_interface
      WHERE     order_number = '-1'
      AND       interface_status = 10;
          
   v_gis_id         gis_interface.gis_id%TYPE;
   n_order_type_id  so_order_types.order_type_id%TYPE;

BEGIN

   FOR rInterface_Program IN cInterface_Program
   LOOP
      v_gis_id := rInterface_Program.GIS_ID;
             
      --lookup correct order_type_id 
      SELECT o.order_type_id 
      INTO   n_order_type_id 
      FROM   so_order_types o,
             gis_interface g
      WHERE  o.name = g.order_type 
      AND    g.gis_id = rInterface_Program.GIS_ID
--      AND    rownum < 2          --justincase ;P
      ;
    
      gis_create_order (rInterface_Program.GIS_ID, 
                        rInterface_Program.ORG_ID,
                        rInterface_Program.FILE_ID,
                        rInterface_Program.CUSTOMER_ID,
                        n_order_type_id,
                        rInterface_Program.CREATED_BY,
                        rInterface_Program.CREATION_DATE);      
   END LOOP;
   
EXCEPTION 
   --if no records found this time, no problem -- don't log, and wait until next concurrent iteration/spin
   WHEN NO_DATA_FOUND THEN            
      NULL;                                     
      
   WHEN OTHERS THEN
   ROLLBACK;
   UPDATE   GIS_INTERFACE
   SET      interface_status = '55'      --'ORDER INTERFACE ERROR'           
            ,status_date = SYSDATE
   WHERE    gis_id = v_gis_id;
   gis_log_event( v_gis_id, '55 - ORDER INTERFACE ERROR', 'gis_order_request', SQLERRM);
   COMMIT;

END gis_order_request;


/*-------------------------------------------------------------------------+
 | PUBLIC PROCEDURE                                                        |
 |    gis_return_oenum                                                     |
 |                                                                         |
 | DESCRIPTION                                                             |
 |    This is a Concurrent Process to retrieve newly-assigned OE#'s from   |
 |    OE Interface tables, and attach the associated file_id's to that     |
 |      HEADER_ID record.                                                  |
 |                                                                         |
 | MODIFICATION HISTORY                                                    |
 |    02-Jun-2000       David Anselmi  Created.                            |
 |    15-Oct-2000       David Anselmi  Exception Handling, Attachment code |
 |    21-Jan-2001       Dave Anselmi   Complete/Installed in AU            |
 |    02-Apr-2001       Patrick Low   Remove references to _all tables     |
 |    22-Jun-2001       Dave Anselmi  Added ROLLBACK to EXCEPTION handling |
 |                                                                         |
 +-------------------------------------------------------------------------*/

PROCEDURE gis_return_oenum (errbuf   OUT VARCHAR2,
                            retcode  OUT VARCHAR2)

IS
   n_header_id               oe_order_headers.HEADER_ID%TYPE;
   v_order_number            VARCHAR2(150);
   n_order_number            oe_order_headers.ORDER_NUMBER%TYPE;
   v_orig_sys_document_ref   oe_order_headers.ORIG_SYS_DOCUMENT_REF%TYPE;
   v_file_id                 oe_order_headers.ORIG_SYS_DOCUMENT_REF%TYPE;
   v_err                     VARCHAR2(1000);
   v_gis_id                  gis_interface.gis_id%TYPE;
   v_interface_status        gis_interface.interface_status%TYPE;

   CURSOR          cParse_GIS_Iface IS
      SELECT       *
      FROM         gis_interface
      WHERE        order_number = '-1'     
      AND          interface_status = 60
      ;

   CURSOR          cParse_OE_Iface (v_orig_sys_doc_ref IN VARCHAR2) IS
      SELECT       header_id, order_number, orig_sys_document_ref
      FROM         oe_order_headers
      WHERE        orig_sys_document_ref = v_orig_sys_doc_ref;

   rParse_OE_Iface   cParse_OE_Iface%ROWTYPE;
   

BEGIN         

   FOR rParse_GIS_Iface IN cParse_GIS_Iface 
   LOOP

      v_gis_id  := rParse_GIS_Iface.gis_id;
      v_file_id := 'GIS - ('||to_char(v_gis_id)||') - '||to_char(rParse_GIS_Iface.file_id);
      v_interface_status := rParse_GIS_Iface.interface_status;

     OPEN    cParse_OE_Iface(v_file_id);
     FETCH   cParse_OE_Iface INTO rParse_OE_Iface;
     IF (cParse_OE_Iface%NOTFOUND) THEN
       rParse_OE_Iface.header_id := NULL;
       rParse_OE_Iface.order_number := NULL;
     END IF;
     CLOSE cParse_OE_Iface;
      
         
     IF ( (rParse_OE_Iface.header_id is not NULL) 
     AND  (rParse_OE_Iface.order_number is not NULL) ) 
     THEN

       IF v_interface_status = 60 THEN
         --put newly-retrieved oe# in gis_interface and update status to 'in-process'
         UPDATE gis_interface
         SET order_number = rParse_OE_Iface.order_number     
            ,interface_status = '70'      --'ORDER INTERFACE COMPLETE'
            ,status_date = SYSDATE
         WHERE  gis_id = rParse_GIS_Iface.gis_id;
         COMMIT;
         gis_log_event( v_gis_id, '70 - ORDER INTERFACE COMPLETE', 'gis_return_oenum',SQLERRM); 
         v_interface_status := 70;
       END IF;

       IF v_interface_status = 70 THEN
         --call attachment api w/ file_id 
         gis_attach_image( rParse_GIS_Iface.last_updated_by      --p_user_id     NUMBER
                           ,rParse_GIS_Iface.file_id             --p_file_id     NUMBER
                           ,rParse_GIS_Iface.gis_id              --p_gis_id      NUMBER
                           ,'OE_ORDER_HEADERS_ALL'               --p_table_name  VARCHAR2
                           ,rParse_OE_Iface.header_id            --p_pk_value    NUMBER
                        );
         gis_log_event( v_gis_id, '71 - DOCUMENT ATTACHED SUCCESSFULLY', 'gis_return_oenum',SQLERRM);
         v_interface_status := 71;
       END IF;

       IF v_interface_status = 71 THEN
         -- update status => 'PRE-INDEXED'
         INSERT INTO gis_doc_status
         SELECT t.document_id, s.doc_status_type_id, sysdate, 1, sysdate, 1
         FROM   fnd_documents_tl t, gis_doc_status_types s, fnd_documents_tl_dfv v
         WHERE  t.doc_attribute_category = 'Order Management'
         AND    t.media_id = rParse_GIS_Iface.file_id 
/** added the following 2 lines per Patrick, 22-Jun-01, DA **/
         AND    t.rowid = v.rowid
         AND    v.primary_document_id = 'Y'
         AND    s.doc_status_name = 'PRE-INDEXED';
         gis_log_event( v_gis_id, '72 - UPDATE GIS_DOC_STATUS -> "PRE-INDEXED"', 'gis_return_oenum',SQLERRM);
         v_interface_status := 72;
       END IF;
      
       IF v_interface_status = 72 THEN
         -- copy the order number to fnd_documents_tl
         -- update status => 'PRE-INDEXED'
         UPDATE fnd_documents_tl 
         SET
            doc_attribute6 = 'PRE-INDEXED',              
            doc_attribute9 = to_char(rParse_OE_Iface.order_number)  
         WHERE rowid = 
            (SELECT t.rowid
             FROM   fnd_documents_tl t, fnd_documents_tl_dfv v
             WHERE  t.rowid = v.rowid
             AND    v.primary_document_id = 'Y'
             AND    t.doc_attribute_category = 'Order Management'
             AND    t.media_id = rParse_GIS_Iface.file_id);
         gis_log_event( v_gis_id, '73 - UPDATE FND_DOCUMENTS_TL', 'gis_return_oenum',SQLERRM);
         v_interface_status := 73;
       END IF;

       IF v_interface_status = 73 THEN
         -- copy the GTN to oe_order_headers
--         update oe_order_headers_dfv
--         set value = ?? where rowid = (
--         select rowid from oe headers
--         where header_id = ??
--         );
--         gis_log_event( v_gis_id, '74 - INSERT OE# INTO OE_ORDER_HEADERS (_DFV)', 'gis_return_oenum',SQLERRM);
         v_interface_status := 74;
       END IF;

       IF v_interface_status = 74 THEN
         --if all previous (bits ;D) updated w/o errors, then
         --update interface_status in gis_interface 
         UPDATE    GIS_INTERFACE
         SET       interface_status = '80'      --'COMPLETE'           
                  ,status_date = SYSDATE
         WHERE    gis_id = rParse_GIS_Iface.GIS_ID;
         gis_log_event( v_gis_id, '80 - DONE.', 'gis_return_oenum',SQLERRM);
       END IF;

      
     END IF;

   END LOOP;


EXCEPTION

   WHEN OTHERS THEN
      ROLLBACK;
      UPDATE    GIS_INTERFACE
      SET       interface_status = '55'      --'ORDER INTERFACE ERROR'           
               ,status_date = SYSDATE
      WHERE    gis_id = v_gis_id;
      gis_log_event( v_gis_id, '55 - ORDER INTERFACE ERROR', 'gis_return_oenum',SQLERRM);

   COMMIT;

END gis_return_oenum;


/*-------------------------------------------------------------------------+
 | PUBLIC PROCEDURE                                                        |
 |    gis_create_order                                                     |
 |                                                                         |
 | DESCRIPTION                                                             |
 |    This INSERTs the appropriate record/request in the OM open iface     |
 |    tables.                                                              |
 |                                                                         |
 | MODIFICATION HISTORY                                                    |
 |    02-Jun-2000       David Anselmi  Created.                            |
 |    15-Oct-2000       David Anselmi  Modified:                           |
 |    21-Jan-2001       Dave Anselmi  Complete/Installed in AU             |
 |    02-Apr-2001       Patrick Low   Remove references to _all tables     |
 |    22-Jun-2001       Dave Anselmi  Added ROLLBACK to EXCEPTION handling |
 |                                                                         |
 +-------------------------------------------------------------------------*/

PROCEDURE gis_create_order( p_gis_id            IN  NUMBER,
                            p_org_id            IN  NUMBER,
                            p_file_id           IN  NUMBER,
                            p_gis_customer_id   IN  NUMBER,
                            p_order_type_id     IN  NUMBER,
                            p_created_by        IN  NUMBER,
                            p_creation_date     IN  DATE) IS

  v_gis_default_user_id    NUMBER(15) := 3;
  v_orig_sys_document_ref  oe_order_headers.ORIG_SYS_DOCUMENT_REF%TYPE;
  v_order_source_id        so_order_sources.order_source_id%TYPE; 
  v_status                 gis_event_log.error_event%TYPE;

BEGIN         

   v_orig_sys_document_ref := 'GIS - ('||to_char(p_gis_id)||') - '||to_char(p_file_id);
   
   SELECT order_source_id
   INTO v_order_source_id
   FROM so_order_sources
   WHERE name = (
      SELECT 'GIS - ' ||name
      FROM hr_operating_units
      WHERE organization_id = p_org_id );

   INSERT INTO oe_headers_iface_all
     (SOLD_TO_ORG_ID
      ,interface_status
      ,error_flag
      ,request_id
      ,operation_code
      ,creation_date
      ,created_by
      ,last_update_date
      ,last_updated_by
      ,orig_sys_document_ref
      ,order_source_id
      ,order_type_id
      ,order_category
--      ,org_id      --removed per OM DEV AKSINGH, bug # 1532099
     )
      
   VALUES
     (p_gis_customer_id        --sold_to_org_id
      ,NULL                    --interface_status
      ,NULL                    --error_flag
      ,NULL                    --request_id
      ,'INSERT'                --operation_code
      ,sysdate                 --creation_date
      ,p_created_by            --created_by
      ,sysdate                 --last_update_date
      ,p_created_by            --last_updated_by
      ,v_orig_sys_document_ref --orig_sys_document_ref
      ,v_order_source_id       --order_source_id...from SO_ORDER_SOURCES.ORDER_SOURCE_ID (apps.oe_order_sources_115)
      ,p_order_type_id         --order_type_id, retrieved from so_order_sources using gis_interface.ORDER_TYPE
      ,NULL                    --order_category
--      ,p_org_id              --org_id
     );

   --update status
   UPDATE    GIS_INTERFACE
   SET       interface_status = '60'      --'ORDER INTERFACE RECORD INSERTED'
   WHERE    gis_id = p_gis_id;           
   gis_log_event( p_gis_id, '60 - ORDER INTERFACE RECORD INSERTED', 'gis_create_order',SQLERRM);
            
   COMMIT;


EXCEPTION
  WHEN OTHERS THEN
   ROLLBACK;
   update   GIS_INTERFACE
   set      interface_status = '55'      --'ORDER INTERFACE ERROR'           
            ,status_date = SYSDATE
   where    gis_id = p_gis_id;
   gis_log_event( p_gis_id, '55 - ORDER INTERFACE ERROR', 'gis_create_order', SQLERRM);

   COMMIT;

END gis_create_order;
              


/*-------------------------------------------------------------------------+
-- from Plow,   from gis_preindex_pkb                                                  

--p_file_id is the media_id - but you can change the code to refer to the
--document_id if you want - I just kept it the same (ie. p_file_id =
--media_id) throughout all the packages/procedure - didn't want to get
--confused with file_id's and document_id's.

 |    22-Jun-2001       Dave Anselmi  Update Attach to handle multi-attach |
 +-------------------------------------------------------------------------*/
  
PROCEDURE gis_attach_image(p_user_id NUMBER,
                       p_file_id     NUMBER,
                       p_gis_id      NUMBER,
                       p_table_name  VARCHAR2,
                       p_pk_value    NUMBER) IS

  CURSOR cEntity IS
    SELECT data_object_code
    FROM   fnd_document_entities
    WHERE  data_object_code = 'OE_ORDER_HEADERS';

  CURSOR cAttach(f_entity VARCHAR2, f_pk VARCHAR2) IS
    SELECT NVL(MAX(seq_num), 0) + 10
    FROM fnd_attached_documents
    WHERE entity_name = f_entity
    AND pk1_value = f_pk; 

  nPKID          NUMBER := NULL;          
  nAttachID      NUMBER := NULL;                              
  nDocID         NUMBER := NULL;
  vEntity        VARCHAR2(240) := NULL;
  nAttach        NUMBER := 10;                
                          

BEGIN
   nPKID := p_pk_value;   

   SELECT fnd_attached_documents_s.nextval
   INTO   nAttachID
   FROM   dual;

   SELECT d.document_id
   INTO   nDocID
   FROM   fnd_documents_tl t, fnd_documents d,  fnd_documents_tl_dfv v
   WHERE  t.media_id = p_file_id    
/** added the following 2 lines per Patrick, 22-Jun-01, DA **/
   AND    t.rowid = v.rowid
   AND    v.primary_document_id = 'Y'
   AND    t.document_id = d.document_id
   AND    d.datatype_id = 
     (SELECT datatype_id
      FROM   fnd_document_datatypes
      WHERE  name = 'FILE'
      AND    NVL(start_date_active, SYSDATE) <= SYSDATE
      AND    NVL(end_date_active, SYSDATE) >= SYSDATE);

   
   OPEN cEntity;
   FETCH cEntity INTO vEntity;
   -- RAISE if not found ???
   CLOSE cEntity;

   IF vEntity IS NOT NULL THEN
   -- Get internal sequence for attached document
   -- DG altered to correctly alocate sequence numbers 11-JAN-2001
      OPEN cAttach(vEntity,nPKID);
      FETCH cAttach INTO nAttach;
      IF cAttach%NOTFOUND THEN
         nAttach := 10;
      END IF;
      CLOSE cAttach;

      INSERT INTO fnd_attached_documents
          (attached_document_id,
           document_id,
           creation_date,
           created_by,
           last_update_date,
           last_updated_by,
           seq_num,
           entity_name,
           pk1_value,
           automatically_added_flag)
      VALUES
          (nAttachID,
           nDocID,
           SYSDATE,
           p_user_id,
           SYSDATE,
           p_user_id,
           nAttach,
           vEntity,
           TO_CHAR(nPKID),
           'N'
      );
   END IF;

   COMMIT;

EXCEPTION
   WHEN OTHERS THEN   
      ROLLBACK;
      UPDATE   GIS_INTERFACE
      SET      interface_status = '55'      --'ORDER INTERFACE ERROR'           
               ,status_date = SYSDATE
      WHERE    gis_id = p_gis_id;
      gis_log_event( p_gis_id, '55 - ORDER INTERFACE ERROR', 'gis_attach_image', SQLERRM);
      COMMIT;

END gis_attach_image;


PROCEDURE gis_log_event( p_gis_id        NUMBER, 
                         p_status        VARCHAR2,   
                         p_object_name   VARCHAR2,
                         p_err           VARCHAR2) IS

BEGIN
   INSERT INTO GIS_EVENT_LOG
     (
      EVENT_ID                  --NOT NULL   NUMBER(15)
      ,FILE_ID                  --NOT NULL   NUMBER(15)
      ,GTN_ID                  --NOT NULL   NUMBER(15)   ...this should be called 'GIS_ID' !!!
      ,OBJECT_NAME            --VARCHAR2(240)
      ,INPUT_PARAMETERS         --VARCHAR2(240)
      ,ERROR_EVENT            --VARCHAR2(240)          ...this should be called 'AUDIT_EVENT' !!!
      ,ERROR_MESSAGE            --VARCHAR2(240)
      ,LAST_UPDATED_BY         --NOT NULL   NUMBER(15)
      ,LAST_UPDATE_DATE         --NOT NULL   DATE
      ,LAST_UPDATE_LOGIN      --NUMBER(15)
      ,CREATED_BY               --NOT NULL   NUMBER(15)
      ,CREATION_DATE            --NOT NULL   DATE
     )
     
   VALUES
     (
      gis_event_log_s.nextval
      ,1
      ,p_gis_id
      ,p_object_name
      ,NULL
      ,p_status
      ,p_err
      ,1
      ,SYSDATE
      ,NULL
      ,1
      ,SYSDATE
      );
      
   COMMIT;

END gis_log_event;


END GIS_OM_INTERFACE;
/

-- the following bits are kept for REFERENCE only.
  
--  select to_number(decode(substr(userenv('CLIENT_INFO'),1,1),' ',null,substr(userenv('CLIENT_INFO'),1,10)))
--  into   v_org_id
--  from   dual;
   


